/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Envelope.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_Envelope_H
#define OpenSOAP_Envelope_H

#include <OpenSOAP/Block.h>
#include <OpenSOAP/ByteArray.h>
#include <OpenSOAP/String.h>

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    typedef struct tagOpenSOAPEnvelope OpenSOAPEnvelope;
    typedef OpenSOAPEnvelope    *OpenSOAPEnvelopePtr;

    int
    OPENSOAP_API
    OpenSOAPEnvelopeCreate(OpenSOAPEnvelopePtr *soap_env);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeCreateMB(const char *soap_ver,
                             const char *env_prefix,
                             OpenSOAPEnvelopePtr *soap_env);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeCreateWC(const wchar_t *soap_ver,
                             const wchar_t *env_prefix,
                             OpenSOAPEnvelopePtr *soap_env);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeCreateCharEncoding(const char *ch_enc,
                                       OpenSOAPByteArrayPtr b_arry,
                                       OpenSOAPEnvelopePtr *soap_env);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeRetain(OpenSOAPEnvelopePtr soap_env);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeRelease(OpenSOAPEnvelopePtr soap_env);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeAddHeaderBlockMB(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                     const char * /* [in] */ block_name,
                                     OpenSOAPBlockPtr * /* [out] */ h_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeAddHeaderBlockWC(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                     const wchar_t * /* [in] */ block_name,
                                     OpenSOAPBlockPtr * /* [out] */ h_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetNextHeaderBlock(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                       OpenSOAPBlockPtr * /* [in, out] */ h_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetHeaderBlockMB(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                     const char * /* [in] */ block_name,
                                     OpenSOAPBlockPtr * /* [out] */ h_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetHeaderBlockWC(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                     const wchar_t * /* [in] */ block_name,
                                     OpenSOAPBlockPtr * /* [out] */ h_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeAddBodyBlockMB(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                   const char * /* [in] */ block_name,
                                   OpenSOAPBlockPtr * /* [out] */ b_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeAddBodyBlockWC(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                   const wchar_t * /* [in] */ block_name,
                                   OpenSOAPBlockPtr * /* [out] */ b_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetNextBodyBlock(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                     OpenSOAPBlockPtr * /* [in, out] */ b_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetBodyBlockMB(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                   const char * /* [in] */ block_name,
                                   OpenSOAPBlockPtr * /* [out] */ b_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetBodyBlockWC(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                   const wchar_t * /* [in] */ block_name,
                                   OpenSOAPBlockPtr * /* [out] */ b_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetCharEncodingString(OpenSOAPEnvelopePtr /* [in] */ soap_env,
                                          const char * /* [in] */ ch_enc,
                                          OpenSOAPByteArrayPtr /* [out] */ b_ary);
    
    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetHeaderCharEncodingString(OpenSOAPEnvelopePtr /* [in] */ soap_env,
											    const char * /* [in] */ ch_enc,
											    OpenSOAPByteArrayPtr /* [out] */ b_ary);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetBodyCharEncodingString(OpenSOAPEnvelopePtr /* [in] */ soap_env,
											  const char * /* [in] */ ch_enc,
											  OpenSOAPByteArrayPtr /* [out] */ b_ary);
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_Envelope_H */
