/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Socket.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#ifndef OpenSOAP_IMPL_Socket_H
#define OpenSOAP_IMPL_Socket_H

#include "Stream.h"

#if defined(NONGCC_WIN32)
#include <winsock.h>
#else
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
    
    typedef
#if defined(NONGCC_WIN32)
    SOCKET
#else
    int
#endif
    SOCKET_TYPE;
    
    typedef struct tagOpenSOAPSocket OpenSOAPSocket;
    typedef OpenSOAPSocket    *OpenSOAPSocketPtr;
    
    struct tagOpenSOAPSocket {
        OpenSOAPStream  super;
        SOCKET_TYPE     socket;
    };

    int
    OPENSOAP_API
    OpenSOAPSocketInitialize(OpenSOAPSocketPtr /* [in, out] */ s,
                             OpenSOAPObjectFreeFunc /* [in] */ free_func,
                             OpenSOAPStreamReadFunc  /* [in] */ read_func,
                             OpenSOAPStreamWriteFunc /* [in] */ write_func);
    
    int
    OPENSOAP_API
    OpenSOAPSocketReleaseMembers(OpenSOAPSocketPtr /* [in] */ s);
    
    int
    OPENSOAP_API
    OpenSOAPSocketCreate(OpenSOAPSocketPtr * /* [out] */ s);
    
    int
    OPENSOAP_API
    OpenSOAPSocketRelease(OpenSOAPSocketPtr /* [in] */ s);
    
    int
    OPENSOAP_API
    OpenSOAPSocketOpen(OpenSOAPSocketPtr /* [in, out] */ s,
                       int /* [in] */ domain,
                       int /* [in] */ type,
                       int /* [in] */ protocol);
    int
    OPENSOAP_API
    OpenSOAPSocketConnect(OpenSOAPSocketPtr /* [in, out] */ s,
                          const struct sockaddr * /* [in] */ serv_addr,
                          size_t /* [in] */ addrlen);
    
    int
    OPENSOAP_API
    OpenSOAPSocketClose(OpenSOAPSocketPtr /* [in] */ s);
    

    int
    OPENSOAP_API
    OpenSOAPSocketRead(OpenSOAPSocketPtr /* [in] */ s,
                       unsigned char * /* [out] */ buf,
                       size_t * /* [in, out] */ buf_sz);
    

    int
    OPENSOAP_API
    OpenSOAPSocketWrite(OpenSOAPSocketPtr /* [in] */ s,
                        const unsigned char * /* [in] */ buf,
                        size_t * /* [in, out] */ buf_sz);
    

    int
    OPENSOAP_API
    OpenSOAPSocketReadToDelm(OpenSOAPSocketPtr /* [in] */ s,
                             const unsigned char * /* [in] */ delm,
                             size_t /* [in] */ delm_sz,
                             OpenSOAPByteArrayPtr /* [out] */ buf);
    
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_IMPL_Socket_H */
