/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: XMLElm.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_IMPL_XMLElm_H
#define OpenSOAP_IMPL_XMLElm_H

#if defined(HAVE_LIBXML2_XMLVERSION_H) 
#include <libxml2/tree.h>
#include <libxml2/parser.h>
#include <libxml2/xmlmemory.h>
#else
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xmlmemory.h>
#endif /* !HAVE_LIBXML2_XMLVERSION_H */

#include <OpenSOAP/XMLElm.h>


#include "XMLNode.h"
#include "XMLAttr.h"
#include "XMLNamespace.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

	struct tagOpenSOAPXMLElm {
		OpenSOAPXMLNode super;                      /* Node               */
		OpenSOAPXMLAttrPtr  xmlAttrList;            /* ex.) EncodingStyle */
		OpenSOAPXMLNamespacePtr definedXMLNsList;   /* ex.) SOAP-ENV      */
	};

	int
	OpenSOAPXMLElmInitialize(/* [in, out] */ OpenSOAPXMLElmPtr elm,
							 /* [in] */       OpenSOAPObjectFreeFunc free_func);

	int
	OpenSOAPXMLElmReleaseMembers(/* [in, out] */ OpenSOAPXMLElmPtr elm);

	int
	OpenSOAPXMLElmRelease(/* [in, out] */ OpenSOAPXMLElmPtr elm);

	int
	OpenSOAPXMLElmAddNsInEncodingString(/* [in] */			xmlDocPtr doc, 
										/* [in] */			OpenSOAPXMLElmPtr elm_child, 
										/* [in, out] */	xmlNodePtr childNode);
   
	int
	OpenSOAPXMLElmAddAttrInEncodingString(/* [in] */		xmlDocPtr doc, 
										  /* [in] */ 		OpenSOAPXMLElmPtr elm_child, 
										  /* [in, out] */	xmlNodePtr childNode);

	int
	OpenSOAPXMLElmConvertEncodingToUTF8Libxml(/* [in] */	const char *name,
											  /* [out] */	OpenSOAPByteArrayPtr utf8_name,
											   /* [out] */	const unsigned char **utf8_name_beg);

	int
	OpenSOAPXMLElmAddNodeWithLibxml(xmlDocPtr doc,
									xmlNodePtr node, 
									OpenSOAPXMLElmPtr elm);

	int
	OpenSOAPXMLElmCreateChildNodeFromLibxml(OpenSOAPXMLElmPtr elm, 
											xmlNodePtr node);

	int
	OpenSOAPXMLElmConvertEncodingFromUTF8Libxml(const xmlChar *name,
												OpenSOAPByteArrayPtr mb_name,
												const unsigned char **mb_name_beg);

	int
	OpenSOAPXMLElmAddAttrInCharEncoding(xmlNodePtr childNode,
										OpenSOAPXMLElmPtr elm_cld);

	int
	OpenSOAPXMLElmAddNsInCharEncoding(xmlNodePtr childNode,
									  OpenSOAPXMLElmPtr elm_cld);

	int
	OPENSOAP_API
	OpenSOAPXMLElmAddAttributeString(OpenSOAPXMLElmPtr elm,
									 OpenSOAPStringPtr attr_name,
									 OpenSOAPStringPtr attr_type,
									 void *attr_value,
									 OpenSOAPXMLAttrPtr *attr);
	int
	OPENSOAP_API
	OpenSOAPXMLElmGetAttributeString(OpenSOAPXMLElmPtr elm,
									 OpenSOAPStringPtr attr_name,
									 OpenSOAPXMLAttrPtr * attr);
	int
	OPENSOAP_API
	OpenSOAPXMLElmGetValueString(OpenSOAPXMLElmPtr elm,
								 OpenSOAPStringPtr type_name,
								 void * value);
	int
	OPENSOAP_API
	OpenSOAPXMLElmAddChildString(OpenSOAPXMLElmPtr elm,
								 OpenSOAPStringPtr cld_name,
								 OpenSOAPXMLElmPtr * cld_elm);
	int
	OPENSOAP_API
	OpenSOAPXMLElmSetValueString(OpenSOAPXMLElmPtr elm,
								 OpenSOAPStringPtr type_name,
								 void * value);
	int
	OPENSOAP_API
	OpenSOAPXMLElmGetChildString(OpenSOAPXMLElmPtr elm,
								 OpenSOAPStringPtr cld_name,
								 OpenSOAPXMLElmPtr * cld_elm);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_IMPL_XMLElm_H */
