/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: IdManager.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if HAVE_CONFIG_H
# include "config.h"
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <cstdio>
#include <unistd.h>
#include <string>

#include <sys/stat.h>

#include "ServerCommon.h"
#include "IdManager.h"

using namespace OpenSOAP;

//for LOG
static const std::string CLASS_SIG = "IdManager";

IdManager::IdManager()
  : idSequence_(0)
{
  socketAddr_ = IDMGR_SOCKET_ADDR; //"//var/tmp/OpenSOAP/SOCKET/IdManager";
}

IdManager::~IdManager()
{
  
}

void IdManager::run()
{
  //===================================================================
  // ʐM 
  //===================================================================
  typedef struct sockaddr SockAddr;
  typedef struct sockaddr_un SockAddrUn;

  int sockfd = int();
  int newsockfd = int();
  SockAddrUn serverAddr;
  SockAddrUn clientAddr;
#if HAVE_TYPE_OF_ADDRLEN_AS_SOCKLEN_T
  socklen_t addrLen = socklen_t(); //socket_t();
  socklen_t clientLen = socklen_t();
#endif
#if HAVE_TYPE_OF_ADDRLEN_AS_SIZE_T
  size_t addrLen = size_t();
  size_t clientLen = size_t();
#endif
#if HAVE_TYPE_OF_ADDRLEN_AS_INT
  int addrLen = int();
  int clientLen = int();
#endif
  
  //DataRetainerƂ̒ʐM|[g
  if (0 > (sockfd = socket(AF_UNIX, SOCK_STREAM, 0))) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::run: open socket failed" << endl;
    //throw std::exception();
    // ONX̐݌vKv 
  }
#ifdef DEBUG
  cerr << CLASS_SIG << "::run: socket open OK!" << endl;
#endif

  memset((char*)&serverAddr, 0, sizeof(serverAddr));
  serverAddr.sun_family = AF_UNIX;
  strcpy(serverAddr.sun_path, socketAddr_.c_str());
  addrLen = sizeof(serverAddr.sun_family) + strlen(serverAddr.sun_path);

  unlink(socketAddr_.c_str());

  mode_t newMask = 0x000;
  mode_t curMask = umask(newMask);

  if (0 > bind(sockfd, (SockAddr*)&serverAddr, addrLen)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::run: bind failed" << endl;
    //throw std::exception();
    // ONX̐݌vKv 
  }

  umask(curMask);

#ifdef DEBUG
  cerr << CLASS_SIG << "::run: bind OK!" << endl;
#endif

  if (0 > listen(sockfd, 5)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::run: listen failed" << endl;
    //throw std::exception();
    // ONX̐݌vKv 
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::run: listen OK!" << endl;
#endif
    
  for (;;) {
    clientLen = sizeof(clientAddr);

#ifdef DEBUG
    cerr << CLASS_SIG << "::run: accept ready!" << endl;
#endif
    
    newsockfd = accept(sockfd, (SockAddr*)&clientAddr, &clientLen);
    if (0 > newsockfd) {
      cerr << TAG_ERR << CLASS_SIG 
	   << "::run: accept failed" << endl;
      //throw std::exception();
      // ONX̐݌vKv 
    }

#ifdef DEBUG
    cerr << CLASS_SIG << "::run: accept OK!" << endl;
#endif

    if (0 > write(newsockfd, getNewId())) {
      cerr << TAG_ERR << CLASS_SIG 
	   << "::run: write failed" << endl;
      //throw std::exception();
      // ONX̐݌vKv 
    }
      
    close(newsockfd);
  }
}


std::string IdManager::getNewId()
{
  //_ł̃VXe𕶎YYYYMMDDHH24MMSS
  //\IDƂB
  char buf[BUFSIZ];
  time_t cur_time;
  time(&cur_time);
  struct tm* cur_tm = localtime(&cur_time);
  //size_t len = strftime(buf, BUFSIZ, "%Y%m%d%H%M%S", cur_tm);
  strftime(buf, BUFSIZ, "%Y%m%d%H%M%S", cur_tm);
  std::string mainId(buf);
  
  //r
  //ꎞłȂ΃V[PXԍZbg
  if (mainId != prevId_) {
    idSequence_ = 0;
  }
  //ێ
  prevId_ = mainId;
  sprintf(buf, "%s%05d", mainId.c_str(), ++idSequence_);
  
#ifdef DEBUG
  cerr << "getNewId: ID=[" << buf << "]" << endl;
#endif
  
  //return DataRetainer::Id(buf);
  return buf;
}


int main()
{
  IdManager idMngr;
  idMngr.run();

  return 0;
}
