/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: MsgAttrHandler.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <string>

#include <sys/socket.h>
#include <sys/un.h>
#include <cstdio>
#include <unistd.h>
#include <string>

#include <stdexcept>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include "MapUtil.h"
#include "ServerCommon.h"
#include "SSMLAttrMgrProtocol.h"
#include "XmlQuery.h"
#include "MsgAttrHandler.h"

using namespace OpenSOAP;

// for LOG
static const std::string CLASS_SIG = "MsgAttrHandler";

MsgAttrHandler::MsgAttrHandler()
  : docPtr_(NULL)
{
  
}

MsgAttrHandler::MsgAttrHandler(const std::string& xmlMsg)
  : docPtr_(NULL)
{
  if (0 != setMsg(xmlMsg)) {
    throw std::exception();
  }
}

MsgAttrHandler::~MsgAttrHandler()
{
  if (docPtr_) {
    xmlFreeDoc(docPtr_);
  }
}

int MsgAttrHandler::setMsg(const std::string& xmlMsg)
{
  /* COMPAT: Do not generate nodes for formatting spaces */
  LIBXML_TEST_VERSION
    xmlKeepBlanksDefault(0);

  docPtr_ = xmlParseMemory((char*)xmlMsg.c_str(), xmlMsg.length());

#if DEBUG
  cerr << "MsgAttrHandler::setMsg" << endl;
  cerr << "xmlMsg = " << endl;
  cerr << xmlMsg << endl;
#endif
  
  if (NULL == docPtr_) {
    cerr << TAG_ERR << CLASS_SIG << "::setMsg: xmlParseMemory returned NULL"
	 << endl;
    return -1;
  }
  /* check document (root node) */
  xmlNodePtr node1 = xmlDocGetRootElement(docPtr_);
  if (node1 == NULL) {
    cerr << TAG_ERR << CLASS_SIG << "::setMsg: empty document"
	 << endl;
    xmlFreeDoc(docPtr_);
    return -1;
  }

  return 0;
}

int MsgAttrHandler::queryXml(const std::string& query, 
			      std::vector<std::string>& values)
{
#ifdef DEBUG
  cerr << CLASS_SIG << "::queryXml() arg=[" << query << "]" << endl;
#endif DEBUG

  XmlQuery* xmlQuery = new XmlQuery(query);
  
  xmlNodePtr node = xmlDocGetRootElement(docPtr_);
  
  int ret = xmlQuery->getValue(docPtr_, node, values);

#ifdef DEBUG
  cerr << "xmlQuery ret=(" << ret << ")" << endl;
#endif DEBUG

  return ret;
}

