/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: connection.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <cstdio>
#include <string>
#include <vector>

#include <fcntl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <unistd.h>

#include "connection.h"


void
sendStdout (std::vector<std::string> soapMsg)
{
  for (std::vector<std::string>::iterator iter = soapMsg.begin();
       iter != soapMsg.end(); iter++) {
    cout << *iter << endl;
  }
}

void
recvStdin (std::vector<std::string>& soapMsg)
{
  const unsigned int bufLength = 10240;
  char buf[bufLength];
  
  while (fgets(buf, bufLength, stdin)) {
//      if (strncmp(buf, "QUIT", 4) == 0) {
//        break;
//      } 
    soapMsg.push_back (std::string (buf));
  }
}

void
sendStdout (std::string soapMsg)
{
  cout << soapMsg << endl;
}

void
recvStdin (std::string& soapMsg)
{
  const unsigned int bufLength = 1024;
  char buf[bufLength];
  
  while (fgets(buf, bufLength, stdin)) {
    soapMsg += std::string(buf);
  }
}


int
openFIFO (std::string fifo_name, int flags)
{
  
  int fd = open (fifo_name.c_str(), flags);
  if (fd < 0) {
    cerr << "FIFO open failed !!" << endl;
  }
  return fd;
}

void
sendFIFO (int fd, std::vector<std::string> soapMsg)
{
  cerr << "sendFIFO" << endl;
  for (std::vector<std::string>::iterator iter = soapMsg.begin();
       iter != soapMsg.end(); iter++) {
    cerr << *iter;
    if (iter == soapMsg.end() - 1) {
      usleep(100000);
    }
    write (fd, iter->c_str(), iter->length());
  }
  fsync(fd);
}


void
recvFIFO (int fd, std::vector<std::string>& soapMsg)
{
  cerr << "FIFO recv begin" << endl;
  fsync(fd);
  std::vector<char>::size_type        bufLength = 1024L;
  std::vector<char>   buf(bufLength, '\0');
  
  std::vector<char>::size_type readLen;
  
  readLen = read(fd, buf.begin(), buf.size());  
  do {
    std::string recv_str (buf.begin(), readLen);
    cerr << "readLen = " << readLen << ": " << recv_str;
    cerr.flush();
    soapMsg.push_back (recv_str);
    fsync(fd);
    readLen = read(fd, buf.begin(), buf.size());  
  } while (readLen > 0);
  fsync(fd);
  
  cerr << "FIFO recv end" << endl;
  
}

/* FIFO  string NX𑗂 */
void
sendFIFO (int fd, std::string soapMsg)
{
  cerr << "sendFIFO" << endl;
  write (fd, soapMsg.c_str(), soapMsg.length());
  fsync(fd);
}

/* FIFO ǂݎʂ string NXɓ */
void
recvFIFO (int fd, std::string& soapMsg)
{
  cerr << "FIFO recv begin" << endl;
  fsync(fd);
  std::vector<char>::size_type        bufLength = 1024L;
  std::vector<char>   buf(bufLength, '\0');
  
  std::vector<char>::size_type readLen;
  
  readLen = read(fd, buf.begin(), buf.size());  
  do {
    std::string recv_str (buf.begin(), readLen);
    cerr << "readLen = " << readLen << ": " << recv_str;
    cerr.flush();
    soapMsg += recv_str;
    fsync(fd);
    readLen = read(fd, buf.begin(), buf.size());  
  } while (readLen > 0);
  fsync(fd);
  
  cerr << "FIFO recv end" << endl;
  
}

int
openSocket (const std::string &hostname, unsigned int port)
{
    int fd;
    struct sockaddr_in    addr;
    struct hostent *hp;
  
    /*
     *  \PbgB̃\PbgUNIXhCŁAXg[^\PbgB
     */
    if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        cerr << "socket is not created." << endl;
	return -1;
    }
  
    /* 
     * addr̒g0ɂĂȂƁAbind()ŃG[N邱Ƃ
     */
  
    memset ((char *)&addr, 0, sizeof(addr));
    
    /*
     * \Pbg̖OĂ
     */
  
    if ((hp = gethostbyname(hostname.c_str())) == NULL) {
        cerr << "No such host." << endl;
	return -1;
    }
    memcpy(reinterpret_cast<char *>(&addr.sin_addr),
	   hp->h_addr, hp->h_length);
    addr.sin_family = AF_INET;
    addr.sin_port = htons(port);
    
    /*
     *  T[o[Ƃ̐ڑ݂Bꂪ邽߂ɂ́AT[o[ł
     *  ̃AhXbind()āAlisten()𔭍sĂȂ΂ȂȂB
     */
    if (connect(fd, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
      cerr << "connecting is not established." << endl;
      return -1;
    }
    
    return fd;
}

ssize_t
sendSocket (int fd, const std::string& msg)
{
    ssize_t ret = write(fd, msg.c_str (), msg.length ());
    return ret;
}

ssize_t
recvSocket (int fd, std::string& msg)
{
  std::vector<char>::size_type	bufLength = 2048L;
  std::vector<char>	buf(bufLength);
  
  ssize_t ret = read(fd, buf.begin(), buf.size());
  if (ret >= 0) {
    std::vector<char>::size_type readLen = ret;
    msg =  std::string(buf.begin(), readLen);
  }
  else {
    cerr << "connection::recvSocket: recvSocket failed !" << endl;
  }
  
  return ret;
}
